#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.Instances;

namespace HeuristicLab.Problems.DataAnalysis {
  [StorableClass]
  [Item("Classification Problem", "A general classification problem.")]
  [Creatable("Problems")]
  public class ClassificationProblem : DataAnalysisProblem<IClassificationProblemData>, IClassificationProblem, IStorableContent,
    IProblemInstanceConsumer<ClassificationData>, IProblemInstanceExporter<ClassificationData> {
    public string Filename { get; set; }

    [StorableConstructor]
    protected ClassificationProblem(bool deserializing) : base(deserializing) { }
    protected ClassificationProblem(ClassificationProblem original, Cloner cloner) : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) { return new ClassificationProblem(this, cloner); }

    public ClassificationProblem()
      : base() {
      ProblemData = new ClassificationProblemData();
    }

    public void Load(ClassificationData data) {
      Name = data.Name;
      Description = data.Description;
      Dataset dataset = new Dataset(data.InputVariables, data.Values);
      ProblemData = new ClassificationProblemData(dataset, data.AllowedInputVariables, data.TargetVariable);
      ProblemData.TrainingPartition.Start = data.TrainingPartitionStart;
      ProblemData.TrainingPartition.End = data.TrainingPartitionEnd;
      ProblemData.TestPartition.Start = data.TestPartitionStart;
      ProblemData.TestPartition.End = data.TestPartitionEnd;
      OnReset();
    }

    public ClassificationData Export() {
      if (!ProblemData.InputVariables.Count.Equals(ProblemData.Dataset.DoubleVariables.Count()))
        throw new ArgumentException("Not all input variables are double variables! (Export only works with double variables)");

      ClassificationData claData = new ClassificationData();
      claData.Name = Name;
      claData.Description = Description;
      claData.TargetVariable = ProblemData.TargetVariable;
      claData.InputVariables = ProblemData.InputVariables.Select(x => x.Value).ToArray();
      claData.AllowedInputVariables = ProblemData.AllowedInputVariables.ToArray();
      claData.TrainingPartitionStart = ProblemData.TrainingPartition.Start;
      claData.TrainingPartitionEnd = ProblemData.TrainingPartition.End;
      claData.TestPartitionStart = ProblemData.TestPartition.Start;
      claData.TestPartitionEnd = ProblemData.TestPartition.End;

      List<List<double>> data = new List<List<double>>();
      foreach (var variable in ProblemData.Dataset.DoubleVariables) {
        data.Add(ProblemData.Dataset.GetDoubleValues(variable).ToList());
      }
      claData.Values = Transformer.Transformation(data);

      return claData;
    }
  }
}
